/*
 * main.c
 *
 *  Created on: 27.04.2009
 *      Author: Alexander Boldyrev
 */

#include "local.h"

void printHelp(char *programName) {
	printf("Use: ");
	if (!programName || strlen(programName) == 0) {
		printf("arch");
	} else {
		size_t i = strlen(programName) - 1;
		while (i > 0 && programName[i] != '\\' && programName[i] != '/') {
			i--;
		}
		programName = &programName[i + 1];
		printf("%s", programName);
	}
	printf(" -[axdl] archName.bar firstFile secondFile .. lastFile\n");
	printf("a - add file to archive\n");
	printf("e - extract file from archive\n");
	printf("d - delete from archive\n");
	printf("l - print information about archive\n");
	printf("v - print version of Barch\n");
	printf("f - fix an archive\n");
}

int main(int argc, char **argv) {
	int i;
	const char *tempFileName;
	unsigned long *offsets;

	if (argc <= 1) {
		printHelp(argv[0]);
	} else if (argv[1][0] != '-' || strlen(argv[1]) != 2) {
		printHelp(argv[0]);
	} else {
		switch (argv[1][1]) {
		case 'a':
			if (argc < 4) {
				printHelp(argv[0]);
				break;
			}

			tempFileName = archMakeTemp(argv[2]);
			if (!tempFileName) {
				printf("Error: Couldn't create temp archive for \"%s\"\n",
						argv[2]);
				return 1;
			}

			if (archCheckFileNames(tempFileName, argc - 2, &argv[2]) != 0) {
				printf("Error: Couldn't validation file names\n");
				return 1;
			}

			for (i = 3; i < argc; i++) {
				if (archAddFile(tempFileName, argv[i]) != 0) {
					printf("Error: Couldn't add file \"%s\"\n", argv[i]);
					return 1;
				}
			}

			if (archMake(argv[2], tempFileName) != 0) {
				printf("Error: Couldn't copy temp archive \"%s\" to \"%s\"\n",
						tempFileName, argv[2]);
				return 1;
			}
			break;
		case 'e':
			if (argc != 3) {
				offsets = archCreateOffsetTable(argv[2], argc - 2, &argv[2]);
				if (!offsets) {
					printf("Error: Couldn't create offset table for files.\n");
					return 1;
				}
				for (i = 3; i < argc; i++) {
					if (archExtractFile(argv[2], argv[i], offsets[i - 2]) != 0) {
						printf("Error: Couldn't extract file \"%s\"\n", argv[i]);
						return 1;
					}
				}
			} else {
				int count;
				char **names;

				// TODO: :)
				offsets = archCreateOffsetTableForAll(argv[2], &count, &names);
				if (!offsets) {
					printf("Error: Couldn't create offset table for files.\n");
					return 1;
				}
				for (i = 0; i < count; i++) {
					if (archExtractFile(argv[2], names[i], offsets[i]) != 0) {
						printf("Error: Couldn't extract file \"%s\"\n",
								names[i]);
						return 1;
					}
				}
			}
			break;
		case 'd':
			if (argc < 4) {
				printHelp(argv[0]);
				break;
			}
			offsets = archCreateOffsetTable(argv[2], argc - 2, &argv[2]);
			if (!offsets) {
				printf("Error: Couldn't create offset table for files.\n");
				return 1;
			}

			tempFileName = archDeleteFiles(argv[2], argc - 2, &argv[2]);
			if (!tempFileName) {
				printf("Error: Couldn't create temp archive for \"%s\"\n",
						argv[2]);
				return 1;
			}

			if (archMake(argv[2], tempFileName) != 0) {
				printf("Error: Couldn't copy temp archive \"%s\" to \"%s\"\n",
						tempFileName, argv[2]);
				return 1;
			}
			break;
		case 'l':
			if (argc != 3) {
				printHelp(argv[0]);
				break;
			}
			archPrintToScreen(argv[2]);
			break;
		case 'f':
			if (argc != 3) {
				printHelp(argv[0]);
				break;
			}

			tempFileName = archMakeTempAndFix(argv[2]);
			if (!tempFileName) {
				printf("Error: Couldn't create temp archive for \"%s\"\n",
						argv[2]);
				return 1;
			}

			if (archMake(argv[2], tempFileName) != 0) {
				printf("Error: Couldn't copy temp archive \"%s\" to \"%s\"\n",
						tempFileName, argv[2]);
				return 1;
			}
			break;
		case 'v':
			printf("Barch version: 1.0-rc\n");
			break;
		case 'h':
		default:
			printHelp(argv[0]);
			break;
		}
	}

	return 0;
}
